.yada {
    position: absolute;
    z-index: 9999;
    transition: left 1s ease-out, top 1s ease-out, bottom 1s ease-out, right 1s ease-out, height 1s ease, width 1s ease;
    height: 90px;
    width: 90px;
    cursor: pointer;
}

.yada > img {
    height: 100%;
    width: 100%;
}

.yada.sleeping {
    .active {
        display: none;
    }
    .sleep {
        display: initial;
    }
}

.yada {
    .active {
        display: initial;
    }
    .sleep {
        display: none;
    }
}

.yada:not(.sleeping) .sleepy_yada {
    display: none;
}

.yada-info .offcanvas-title {
    width: 100%;
}

.yada-info .offcanvas-title .row {
    flex-wrap: nowrap;
    margin: auto;
    justify-content: center;
}

.yada-info {
    z-index: 10001;
}

.yada-info .previous, .yada-info .next {
    display: none;
}

.yada-info.offcanvas-start,  .yada-info.offcanvas-end {
    width: 200px;
}

.yada-info.offcanvas-top,  .yada-info.offcanvas-bottom {
    height: fit-content;
    max-height: 50vh;
}

.yada-info .offcanvas-header {
    padding-left: 1rem;
    padding-right: 1rem;
    padding-top: 1rem;
    padding-bottom: 0;
    padding: ""
}

.yada.sleeping {
    right: 30px;
    top: 5px;
    height: 50px;
    width: 50px;
}

.highlighting {
    box-shadow: 0px 0px 15px yellow !important;
    filter: brightness(0.8) !important;
}

.data > .no_message, .no_data > .data_message {
    display: none;
}

.no_data > .no_message, .data > .data_message {
    display: initial;
}


.hidden {
    display: none;
}

input.highlighting {
    border: 5px yellow;

}

button.hightlighting {
    border: 5px yellow;
}


.popover {
  max-width: 350px;
  padding-bottom: 10px
}